using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Diagnostics;

namespace DDRAnalyzer
{
    public enum Version
    {
        None,
        DDR1st,
        DDR2nd,
        DDR3rd,
        DDR4th,
        DDR5th,
        DDRMAX,
        DDRMAX2,
        DDRExtreme,
        DDRSuperNova,
        DDRSuperNova2,
        DDRX,
        DDRX2,
        DDRX3
    }

    public enum Difficulty
    {
        Basic,
        Difficult,
        Expert,
        Challenge
    }

    public struct BPM
    {
        public int min;
        public int max;
    }

    public struct Notes
    {
        public int normal;
        public int freeze;
        public int shock;
    }

    // 
    public struct Music
    {
        public Difficulty difficulty;
        public int level;
        public Notes notes;
        public string property; // 

        public Song song;   // eQ

        public string GetDifficultyString()
        {
            switch (difficulty)
            {
                case Difficulty.Basic:
                    return "(y)";
                case Difficulty.Difficult:
                    return "(x)";
                case Difficulty.Expert:
                    return "()";
                case Difficulty.Challenge:
                    return "(S)";
            }
            return "";
        }

        public int GetWeightedNotes()
        {
            return notes.normal;
        }
    }

    // y
    public class Song 
    {
        public const int MAX_MUSICS = 4;

        public string name;        // Ȗ(HPŎgp)
        public List<string> nameAlias; 
        public string artist;
        public BPM bpm;         // \BPM
        public BPM actualBpm;   // ۂBPM
        public Version version;
        public bool delete;     // 폜
        public Music[] musics;

        public Song()
        {
            name = null;
            nameAlias = null;
            musics = new Music[MAX_MUSICS];

            for (int i = 0; i < MAX_MUSICS; i++)
            {
                musics[i].song = this;
            }
        }

        public int getMusicIndex(string difficulty)
        {
            switch (difficulty)
            {
                case "(y)":
                    return 0;
                case "(x)":
                    return 1;
                case "()":
                    return 2;
                case "(S)":
                    return 3;
            }

            Debug.Assert(false);
            return -1;
        }

        public bool isActualBpmSame()
        {
            return actualBpm.min == bpm.min && actualBpm.max == bpm.max;
        }

        public int getMaxLevel()
        {
            int max = 0;

            foreach (Music m in musics)
            {
                if (max < m.level) max = m.level;
            }
            return max;
        }

        public bool propertyContains(string p)
        {
            foreach (Music m in musics)
            {
                if (m.level != 0)
                {
                    if (m.property!=null && m.property.Contains(p)) return true;
                }
            }
            return false;
        }
    }

    // yȃo[W\[gNX
    public class SongVersionComparer : IComparer
    {
        public int Compare(object x, object y)
        {
            Song song1 = (Song)x;
            Song song2 = (Song)y;

            if ( song2.version != song1.version ) return (int)song2.version - (int)song1.version;
            else                                  return string.Compare(song1.name, song2.name);
        }
    }

    // yȖ\[gNX
    public class SongNameComparer : IComparer
    {
        public int Compare(object x, object y)
        {
            Song song1 = (Song)x;
            Song song2 = (Song)y;

            return string.Compare( song1.name, song2.name );
        }
    }

    // A[eBXg\[gNX
    public class SongArtistComparer : IComparer
    {
        public int Compare(object x, object y)
        {
            Song song1 = (Song)x;
            Song song2 = (Song)y;

            int ret = string.Compare(song1.artist, song2.artist );
            if (ret != 0) return ret;
            else return string.Compare(song1.name, song2.name);
        }
    }

    // BPM\[gNX
    public class SongBPMComparer : IComparer
    {
        public int Compare(object x, object y)
        {
            Song song1 = (Song)x;
            Song song2 = (Song)y;

            if (song2.actualBpm.max != song1.actualBpm.max) return song2.actualBpm.max - song1.actualBpm.max;
            else return string.Compare(song1.name, song2.name);
        }
    }

    // Level\[gNX(eՓx̍őLV)
    public class LevelComparer : IComparer
    {
        public int Compare(object x, object y)
        {
            Song song1 = (Song)x;
            Song song2 = (Song)y;

            int lv1 = song1.getMaxLevel();
            int lv2 = song2.getMaxLevel();

            if (lv2 != lv1) return lv2 - lv1;
            else return string.Compare(song1.name, song2.name);
        }
    }

    // NOTES\[gNX
    public class MusicNotesComparer : IComparer
    {
        public int Compare(object x, object y)
        {
            Music music1 = (Music)x;
            Music music2 = (Music)y;

            if (music2.notes.normal != music1.notes.normal) return music2.notes.normal - music1.notes.normal;
            if (music2.notes.freeze != music1.notes.freeze) return music2.notes.freeze - music1.notes.freeze;
            if (music2.notes.shock != music1.notes.shock) return music2.notes.shock - music1.notes.shock;
            else return string.Compare(music1.song.name, music2.song.name);
        }
    }
}
